package org.processmining.security.plugins.criterion;

import java.text.ParseException;
import java.util.Collection;
import java.util.Date;

import org.processmining.partialorder.models.palignment.Move;
import org.processmining.partialorder.models.palignment.PAlignment;
import org.processmining.security.plugins.Utility;
import org.processmining.security.plugins.replayer.CompositeMove;
import org.processmining.security.plugins.replayer.InterLevelSyncReplayResult;
import org.processmining.security.plugins.ui.CriterionConfigurationPanel.AttributeType;

public class Criteria implements CriteriaInterface{
	private String traceEventAttribute, systemEventAttribute, operation;
	private AttributeType attrType;
	public Criteria(String traceEventAttribute, String systemEventAttribute, String operation, AttributeType attrType){
		this.traceEventAttribute = traceEventAttribute;
		this.systemEventAttribute = systemEventAttribute;
		this.operation = operation;
		this.attrType = attrType;
	}
	public boolean isValid(PAlignment pAlignment, InterLevelSyncReplayResult interLevelAlignment, CompositeMove compositeMove) throws ParseException {
		if(compositeMove.getSystemEvent() == null || compositeMove.getAlignmentMove() == null)
			return true;
		String systemEventAttributeValue = compositeMove.getSystemEvent().getAttributes().get(systemEventAttribute).toString();
		String traceEventAttributeValue = "";
		
		if(compositeMove.getAlignmentMove().isModelMove()){
			if(traceEventAttribute.equals("concept:name"))
				traceEventAttributeValue = compositeMove.getAlignmentMove().getTransition().toString();		
		}else
			traceEventAttributeValue = compositeMove.getAlignmentMove().getEvent().getAttributes().get(traceEventAttribute).toString();
		
		switch(operation){
			case "=" :
				return systemEventAttributeValue.equalsIgnoreCase(traceEventAttributeValue);		
			case "<=":	
				if(attrType == AttributeType.Date){
					Date traceEventT = null;				
					if(compositeMove.getAlignmentMove().isModelMove())
						traceEventT = estimateStartTime(pAlignment, interLevelAlignment, 
								pAlignment.getDirectParents(compositeMove.getAlignmentMove()), new Date(Long.MIN_VALUE));
					else{						
						traceEventT = Utility.getTime(compositeMove.getAlignmentMove().getEvent(), traceEventAttribute);
					}
					Date systemEventT = Utility.getTime(compositeMove.getSystemEvent(), systemEventAttribute);
					return traceEventT.before(systemEventT) || traceEventT.equals(systemEventT);
				}else if(attrType == AttributeType.Double)
					return Double.valueOf(traceEventAttributeValue) <= Double.valueOf(systemEventAttributeValue) ? true : false;		
				break;
			case ">=":
				if(attrType == AttributeType.Date){
					Date traceEventT = null;				
					if(compositeMove.getAlignmentMove().isModelMove())
						traceEventT = estimateCompleteTime(pAlignment, interLevelAlignment, 
								pAlignment.getDirectChildren(compositeMove.getAlignmentMove()), new Date(Long.MAX_VALUE));
					else
						traceEventT = Utility.getTime(compositeMove.getAlignmentMove().getEvent(), traceEventAttribute);
					Date systemEventT = Utility.getTime(compositeMove.getSystemEvent(), systemEventAttribute);
					return traceEventT.after(systemEventT) || traceEventT.equals(systemEventT);
				}else if(attrType == AttributeType.Double)
					return Double.valueOf(traceEventAttributeValue) >= Double.valueOf(systemEventAttributeValue) ? true : false;
		}
		return false;
	}
	private Date estimateStartTime(PAlignment alignment, InterLevelSyncReplayResult interLevelAlignment, Collection<Move> parentMoves, Date maxDate){
		if(parentMoves.isEmpty())
			return new Date(Long.MIN_VALUE);
		for (Move move : parentMoves) {
			Date parentDate = null;
			if(move.isModelMove()){
				for (CompositeMove cm : interLevelAlignment.getCompositeMoves()) 
					if(cm.getAlignmentMove() != null && cm.getAlignmentMove().equals(move) && cm.getSystemEvent() !=null)
						parentDate = Utility.getTime(cm.getSystemEvent(), "time:timestamp"); 
				if(parentDate == null)
					parentDate = estimateStartTime(alignment, interLevelAlignment, alignment.getDirectParents(move), maxDate);
			}else
				parentDate = Utility.getTime(move.getEvent(), "complete");  
			if(maxDate.before(parentDate))
				maxDate = parentDate;
		}
		return maxDate;
	}
	
	private Date estimateCompleteTime(PAlignment alignment, InterLevelSyncReplayResult interLevelAlignment, Collection<Move> childrenMoves, Date maxDate){
		if(childrenMoves.isEmpty())
			return new Date(Long.MAX_VALUE);
		for (Move move : childrenMoves) {
			Date childrenDate = null;
			if(move.isModelMove())
				childrenDate = estimateCompleteTime(alignment, interLevelAlignment, alignment.getDirectChildren(move), maxDate);
			else
				childrenDate = Utility.getTime(move.getEvent(), "start");
			if(childrenDate.before(maxDate))
				maxDate = childrenDate;
		}
		return maxDate;
	}
}