package org.processmining.security.plugins.heuristic;

import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.partialorder.models.palignment.Move;
import org.processmining.partialorder.models.palignment.PAlignment;
import org.processmining.security.plugins.Utility;
import org.processmining.security.plugins.criterion.CriterionFunction;
import org.processmining.security.plugins.interDataProcessPolicy.CRUD;
import org.processmining.security.plugins.interDataProcessPolicy.CRUDEntry;
import org.processmining.security.plugins.replayer.CompositeMove;
import org.processmining.security.plugins.replayer.CompositeMoveType;
import org.processmining.security.plugins.replayer.InterLevelSyncReplayResult;
import org.processmining.security.plugins.replayer.aStar.CostFunction;

public class HeuristicFunctionImp extends HeuristicFunction{
	private HashMap<XEvent, Double> minCostMap;
	private HashSet<XEvent> systemEventsWithPotentialLink;
	public HeuristicFunctionImp(PAlignment pAlignment, XTrace systemTrace, CostFunction cost, CRUD crud, CriterionFunction criterion) throws ParseException{		
		minCostMap = new HashMap<XEvent, Double>();
		systemEventsWithPotentialLink = new HashSet<XEvent>();
		findPossibleMapping(pAlignment, systemTrace, cost, crud, criterion);
	}
	public double getDistanceToTarget(XTrace systemTrace, int index) {
		double h = 0;
		if(index != -1)
			for(int i = index; i < systemTrace.size(); i++)
				h += minCostMap.get(systemTrace.get(i));
		return h;
	}
	public void findPossibleMapping(PAlignment pAlignment, XTrace systemTrace, CostFunction cost, CRUD crud, CriterionFunction criterionFunction) throws ParseException{
		minCostMap = new HashMap<XEvent, Double>();
		systemEventsWithPotentialLink = new HashSet<XEvent>();
		for (XEvent xEvent : systemTrace) {
			minCostMap.put(xEvent, cost.getCompositeMoveCost(CompositeMoveType.NO_CONTEXT, null));
		}
		InterLevelSyncReplayResult interLevelAlignment = new InterLevelSyncReplayResult("");
		for(Move move : pAlignment.getMoves()){			
			for (XEvent systemEvent : systemTrace) {
				CRUDEntry crudEntry = crud.crudCheck(Utility.getActivity(move), systemEvent);
				CompositeMove compositeMove = new CompositeMove(systemEvent, move, crudEntry);
				Double moveCost = cost.getCompositeMoveCost(compositeMove.getIDPStepType(), move);
				if(criterionFunction.isValid(pAlignment, interLevelAlignment, compositeMove)){
					systemEventsWithPotentialLink.add(systemEvent);
					if(moveCost < minCostMap.get(systemEvent))
						minCostMap.put(systemEvent, moveCost);
				}
			}
		}
	}
}
