package org.processmining.security.plugins.replayer;

import org.deckfour.xes.model.XEvent;
import org.processmining.partialorder.models.palignment.Move;
import org.processmining.security.plugins.Utility;
import org.processmining.security.plugins.interDataProcessPolicy.CRUDEntry;





public class CompositeMove implements Comparable<CompositeMove>{
	
	private XEvent systemEvent;
	private CRUDEntry crudEntry;
	private Move alignmentMove;
	private CompositeMoveType stepType;
	
	public CompositeMove(XEvent systemEvent, Move alignmentMove, CRUDEntry crudEntry){
		this.systemEvent = systemEvent;
		this.alignmentMove = alignmentMove;
		this.crudEntry = crudEntry;
		updateIDPStepTypes();
	}
	public int compareTo(CompositeMove o) {
		// TODO Auto-generated method stub
		return 0;
	}
	public XEvent getSystemEvent() {
		return systemEvent;
	}
	public Move getAlignmentMove() {
		return alignmentMove;
	}
	public CompositeMoveType getIDPStepType() {
		return stepType;
	}
	public CRUDEntry getCRUDEntry(){
		return crudEntry;
	}
	public void setCRUDEntry(CRUDEntry crudEntry) {
		this.crudEntry = crudEntry;
	}
	public void updateIDPStepTypes(){
		if(alignmentMove == null){
			stepType = CompositeMoveType.NO_CONTEXT;
		}else if(alignmentMove.isSyncMove()){
			if(crudEntry == null)
			{
				if(systemEvent == null)
					stepType = CompositeMoveType.SYNC_PROC_NO_OP;
				else
					stepType = CompositeMoveType.SYNC_PROC_MLOG_DATA;
			}else {
				if(systemEvent == null)
					stepType = CompositeMoveType.SYNC_PROC_MMODEL_DATA;
				else
					stepType = CompositeMoveType.SYNC_PROC_SYNC_DATA;
			}
		}else if(alignmentMove.isLogMove()){
			if(crudEntry == null)
			{
				if(systemEvent == null)
					stepType = CompositeMoveType.MLOG_PROC_NO_OP;
				else
					stepType = CompositeMoveType.MLOG_PROC_MLOG_DATA;
			}else {
				if(systemEvent == null)
					stepType = CompositeMoveType.MLOG_PROC_MMODEL_DATA;
				else
					stepType = CompositeMoveType.MLOG_PROC_SYNC_DATA;
			}
		}else if(alignmentMove.isModelMove()){
			if(crudEntry == null)
			{
				if(systemEvent == null)
					stepType = CompositeMoveType.MMODEL_PROC_NO_OP;
				else
					stepType = CompositeMoveType.MMODEL_PROC_MLOG_DATA;
			}else {
				if(systemEvent == null)
					stepType = CompositeMoveType.MMODEL_PROC_MMODEL_DATA;
				else
					stepType = CompositeMoveType.MMODEL_PROC_SYNC_DATA;
			}
		}
	}
	public String toString() {
		String str = (systemEvent == null) ? "(>>, " : "(" + Utility.getAttributeValue(systemEvent, "concept:name") + ", ";
		str += (crudEntry == null) ? ">>, " : crudEntry.toString() + ", ";
		String sysEventName = ">>, ", transName = ">>)";
		if(alignmentMove != null){
			sysEventName = (alignmentMove.getEvent() == null) ? ">>, " : Utility.getAttributeValue(alignmentMove.getEvent(), "concept:name") + ", ";
			transName = (alignmentMove.isLogMove()) ? ">>)" : alignmentMove.getTransition().toString() + ")";
		}
		str += sysEventName + transName;
		return str;
	}
	public boolean hasMissingAction(){
		if(systemEvent == null && crudEntry != null)
			return true;
		return false;
	}
	public boolean hasUnauthorizedAction(){
		if(systemEvent != null && crudEntry == null)
			return true;
		return false;
	}
	public boolean hasAllowedAction(){
		if(systemEvent != null && crudEntry != null)
			return true;
		return false;
	}
	public String getAction(){
		if(crudEntry != null)
			return crudEntry.toString();
		if(systemEvent != null)
			return Utility.getAttributeValue(systemEvent, "concept:name");
		return "";
	}
}
