package org.processmining.security.plugins.replayer;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.connection.PAlignmentResultConnection;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.partialorder.models.palignment.PAlignment;
import org.processmining.partialorder.models.replay.POSyncReplayResult;
import org.processmining.partialorder.plugins.replay.POAlignmentBuilder;
import org.processmining.partialorder.ptrace.model.PLog;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayer.algorithms.costbasedcomplete.CostBasedCompleteParam;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;
import org.processmining.security.plugins.Utility;
import org.processmining.security.plugins.interDataProcessPolicy.CRUD;

public class MultiThreadInterLevelReplayer {		
	public InterLevelReplayResult perform(UIPluginContext context, PNRepResult alignments, XLog systemEventLog, 
			CRUD crud, InterLevelReplayerParameters replayerParam) throws Exception{
		InterLevelReplayer.counter = 0;
		InterLevelReplayResult list = new InterLevelReplayResult();		
		int maxConcurrentThreads = Runtime.getRuntime().availableProcessors();
		context.getProgress().setMaximum(systemEventLog.size());
		context.getProgress().setValue(0);
		
		PAlignmentResultConnection conn = context.getConnectionManager().getFirstConnection(PAlignmentResultConnection.class, context, alignments);
		PLog poLog = conn.getObjectWithRole(PAlignmentResultConnection.PARTIALLOG);
		PetrinetGraph graph = conn.getObjectWithRole(PAlignmentResultConnection.PN);
		TransEvClassMapping map = conn.getObjectWithRole(PAlignmentResultConnection.MAPPING);
		XLogInfo info = XLogInfoFactory.createLogInfo(poLog.getXLog(), map.getEventClassifier());
		CostBasedCompleteParam param = conn.getObjectWithRole(PAlignmentResultConnection.ALGPARAMETER);
		
		Utility.ev2TrMapping = reverse(map);
		Utility.eventClasses = info.getEventClasses();
		Utility.tr2EvMapping = map;
		
		if(!isCRUDValid(crud)){
			String msg = "";
			for (String activity : crud.getActivities()) {
				if(Utility.eventClasses.getByIdentity(activity) == null)
					msg += activity + ", ";
			}
			if(msg.length() > 0)
				msg = msg.substring(0, msg.length() - 2);
			JOptionPane.showMessageDialog(
					new JPanel(),
					"There is no event corresponding to these transitions (event classes): " + msg,
					"CRUD Validation", JOptionPane.WARNING_MESSAGE);
		}
		ThreadPoolExecutor pool = new ThreadPoolExecutor(maxConcurrentThreads, maxConcurrentThreads, 60, TimeUnit.SECONDS,new LinkedBlockingQueue<Runnable>());
		pool.allowCoreThreadTimeOut(true);
		for (SyncReplayResult syncReplayResult : alignments) {
			for (Integer index : syncReplayResult.getTraceIndex()) {
				PAlignment pAlignment = POAlignmentBuilder.computePAlignmentAndUpdateResult(graph, param.getInitialMarking(), param.getFinalMarkings(), info.getEventClasses(), map, 
						poLog.getXLog().get(index), poLog.get(index), index, (POSyncReplayResult) syncReplayResult);
				XTrace systemTrace = systemEventLog.get(index);	
				InterLevelReplayer replayer = new InterLevelReplayer(context, list, pAlignment, systemTrace, crud, replayerParam);
				pool.execute(replayer);
			}
		}
		
		pool.shutdown();
		pool.awaitTermination(10, TimeUnit.HOURS);
		System.out.println("All inter-level alignments have been constructed");		
		return list;
	}
	public boolean isCRUDValid(CRUD crud){
		for (String activity : crud.getActivities()) {
			if(Utility.eventClasses.getByIdentity(activity) == null && !isInvisible(activity)){
				return false;
			}
		}
		return true;
	}
	public boolean isInvisible(String activity){
		for (Transition transition : Utility.tr2EvMapping.keySet()) {
			if(transition.getLabel().equals(activity) && transition.isInvisible())
				return true;
		}
		return false;
	}
	public InterLevelReplayResult perform(UIPluginContext context, List<PAlignment> alignments, XLog systemEventLog, 
			CRUD crud, InterLevelReplayerParameters replayerParam) throws Exception{
		InterLevelReplayer.counter = 0;
		InterLevelReplayResult list = new InterLevelReplayResult();
		int maxConcurrentThreads = Runtime.getRuntime().availableProcessors() ;
		
		ThreadPoolExecutor pool = new ThreadPoolExecutor(maxConcurrentThreads, maxConcurrentThreads, 60, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>());
		pool.allowCoreThreadTimeOut(true);
		context.getProgress().setMaximum(alignments.size());
		context.getProgress().setValue(0);
		
		for (PAlignment pAlignment : alignments) {
			XTrace systemTrace = systemEventLog.get(pAlignment.getTraceIndex());	
			InterLevelReplayer replayer = new InterLevelReplayer(context, list, pAlignment, systemTrace, crud, replayerParam);
			pool.execute(replayer);
		}
		pool.shutdown();
		pool.awaitTermination(10, TimeUnit.HOURS);
		System.out.println("All inter-level alignments have been constructed");	
		return list;
	}
	public static <K,V> HashMap<V,K> reverse(Map<K,V> map) {
	    HashMap<V,K> rev = new HashMap<V, K>();
	    for(Map.Entry<K,V> entry : map.entrySet())
	        rev.put(entry.getValue(), entry.getKey());
	    return rev;
	}
}
