package org.processmining.security.plugins.visualization.crud;

import java.util.HashMap;
import java.util.List;

import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.partialorder.models.palignment.Move;
import org.processmining.security.plugins.Constants;
import org.processmining.security.plugins.Utility;
import org.processmining.security.plugins.interDataProcessPolicy.CRUD;
import org.processmining.security.plugins.replayer.CompositeMove;
import org.processmining.security.plugins.replayer.InterLevelReplayResult;
import org.processmining.security.plugins.replayer.InterLevelResultConnection;
import org.processmining.security.plugins.replayer.CompositeMoveType;
import org.processmining.security.plugins.replayer.InterLevelSyncReplayResult;

public class CRUDProjectionDataSummary {
	private InterLevelResultConnection conn;
	private CRUD crud;
	private InterLevelReplayResult replayResult;
	private String noContext = "Out of context";
	
	private HashMap<String, int[]> summaryMap;	 
			
	public CRUDProjectionDataSummary(PluginContext context, InterLevelReplayResult replayResult) throws ConnectionCannotBeObtained{
		this.replayResult = replayResult;
		conn = context.getConnectionManager().getFirstConnection(InterLevelResultConnection.class, context);
		crud = conn.getObjectWithRole(InterLevelResultConnection.CRUD);
		summaryMap = new HashMap<String, int[]>();
	}
	public void updateSummary(String dataAttribute, AlignmentMoveType alignmentMoveType){
		summaryMap.clear();
		for(String activity : crud.getActivities())
			summaryMap.put(activity, new int[12]);
		summaryMap.put(noContext, new int[12]);
		
		for (InterLevelSyncReplayResult syncReplayResult : replayResult) {
			for (CompositeMove cm : syncReplayResult.getCompositeMoves()) {
				String act = noContext;
				if(cm.getAlignmentMove() != null)
					act = Utility.getActivity(cm.getAlignmentMove());
				
				String dataObject = "", operation = "";
				if(cm.getCRUDEntry() != null){
					dataObject = cm.getCRUDEntry().getData();
					operation = cm.getCRUDEntry().getOperation();
				}else if(cm.getSystemEvent() != null){
					dataObject = Utility.getAttributeValue(cm.getSystemEvent(), Constants.DATA);
					operation = Utility.getAttributeValue(cm.getSystemEvent(), Constants.OPERATION);
				}
				if(satisfyCriterion(dataAttribute, alignmentMoveType, dataObject, operation, cm.getAlignmentMove())){
					int index = getIndex(cm.getIDPStepType(), operation);
					summaryMap.get(act)[index]++;
				}
			}
		}
	}
	public List<String> getDataObjects(){
		 return crud.getDataObjects();
	}
	public HashMap<String, int[]> getSummaryMap() {
		return summaryMap;
	}
	public List<String> getOperations(){
		return crud.getOperations();
	}
	private boolean satisfyCriterion(String dataAttribute, AlignmentMoveType alignmentMoveType, String dataObject, String operation, Move move){
		
		if(dataObject.equals("") && operation.equals(""))
			return false;
		if(!dataAttribute.equalsIgnoreCase(CRUDProjectionPanel.AllDataObjects) && !dataAttribute.equalsIgnoreCase(dataObject))
			return false;
		if(alignmentMoveType != AlignmentMoveType.ALL){
			if(move == null)
				return false;
			if(alignmentMoveType == AlignmentMoveType.MODEL_MOVE && !move.isModelMove())
				return false;
			if(alignmentMoveType == AlignmentMoveType.LOG_MOVE && !move.isLogMove())
				return false;
			if(alignmentMoveType == AlignmentMoveType.SYNCH_MOVE && !move.isSyncMove())
				return false;
		}
		return true;
	}
	public enum AlignmentMoveType{
		ALL, MODEL_MOVE, LOG_MOVE, SYNCH_MOVE
	}
	private int getIndex(CompositeMoveType stepType, String opr){
		int index = -1;
		for(int i = 0; i < crud.getOperations().size(); i++){
			if(opr.equalsIgnoreCase(crud.getOperations().get(i)))
				index = i * 3;
		}
		if(index == -1){
			System.err.println("operation is not supported");
			return -1;
		}
		
		if(isLegitimateAction(stepType))
			return index;
		else if(isIllegitimateAction(stepType))
			return index + 1;
		else if(isMissingAction(stepType))		
			return index + 2;
		return -1;
	}
	private boolean isMissingAction(CompositeMoveType step){
		if(step.equals(CompositeMoveType.MLOG_PROC_MMODEL_DATA) || step.equals(CompositeMoveType.MMODEL_PROC_MMODEL_DATA) || 
				step.equals(CompositeMoveType.SYNC_PROC_MMODEL_DATA))
			return true;
		return false;
	}
	private boolean isIllegitimateAction(CompositeMoveType step){
		if(step.equals(CompositeMoveType.MLOG_PROC_MLOG_DATA) || step.equals(CompositeMoveType.MMODEL_PROC_MLOG_DATA) || 
				step.equals(CompositeMoveType.SYNC_PROC_MLOG_DATA) || step.equals(CompositeMoveType.NO_CONTEXT) || 
				step.equals(CompositeMoveType.MLOG_PROC_SYNC_DATA))
			return true;
		return false;
	}
	private boolean isLegitimateAction(CompositeMoveType step){
		if(step.equals(CompositeMoveType.SYNC_PROC_SYNC_DATA) || step.equals(CompositeMoveType.MMODEL_PROC_SYNC_DATA))
			return true;
		return false;
	}
}
