package org.processmining.security.plugins.visualization.crud;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.framework.util.ui.widgets.ProMScrollPane;
import org.processmining.framework.util.ui.widgets.ProMSplitPane;
import org.processmining.framework.util.ui.widgets.ProMTable;
import org.processmining.framework.util.ui.widgets.WidgetColors;
import org.processmining.security.plugins.visualization.crud.CRUDProjectionDataSummary.AlignmentMoveType;

import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

import info.clearthought.layout.TableLayout;

public class CRUDProjectionPanel extends JPanel implements ActionListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	public static String AllDataObjects = "ALL";
	private CRUDProjectionDataSummary data;
	private ProMComboBox<String> dataObjectComboBox;
	private ProMComboBox<AlignmentMoveType> alignmentMoveTypeComboBox;
	private SummaryTableModel tableModel;
	
	public CRUDProjectionPanel(PluginContext context, CRUDProjectionDataSummary data){
		this.data = data; 
		TableLayout mainLayout = new TableLayout(new double[][] { { TableLayout.FILL }, { TableLayout.FILL } });
		setLayout(mainLayout);
		setBorder(BorderFactory.createEmptyBorder());
//		if (context.getProgress() != null) {
//			context.getProgress().setMaximum(replayResult.size());
//		}
//		setLayout(new GridBagLayout());
		add(createSummaryPanel(data), "0,0");
	}
	private Component createSummaryPanel(final CRUDProjectionDataSummary data) {

		// coloring scheme
		Color bgColor = new Color(30, 30, 30);
		
//		final ProMPropertiesPanel logAlignmentPanel = new ProMPropertiesPanel("Summary");
//		JPanel allAlignmentPanel = new JPanel();
//		allAlignmentPanel.setLayout(new BoxLayout(allAlignmentPanel, BoxLayout.Y_AXIS));
		
		
//		Vector<String> columnIds = prepareNullSafeColumnNames(header);
//		previewTableModel = new BatchUpdateDefaultTableModel(columnIds, 0);
	
		

		
		int lineNumberRight = 0; 
		JPanel statisticPanel = new JPanel();
		
		statisticPanel.setBackground(bgColor);
		double[][] rightMainPanelSize = new double[][] {
				{ TableLayout.PREFERRED },
				{ TableLayout.PREFERRED, 30, TableLayout.PREFERRED, 30, TableLayout.PREFERRED, TableLayout.PREFERRED,
						60, TableLayout.PREFERRED } };

		statisticPanel.setLayout(new TableLayout(rightMainPanelSize));

		// add LEGEND
		statisticPanel.add(createLegendPanel(), "0, " + lineNumberRight++ + ", c, t");
		// MAIN PANEL
		ProMSplitPane splitPanel = new ProMSplitPane(ProMSplitPane.HORIZONTAL_SPLIT);
		splitPanel.setBorder(BorderFactory.createEmptyBorder());
		splitPanel.setOneTouchExpandable(true);
		
//		statisticPanel.setMaximumSize();
		statisticPanel.setPreferredSize(new Dimension(splitPanel.getWidth() / 6, splitPanel.getHeight()));
		//Table
		tableModel = new SummaryTableModel(data.getOperations(), data.getSummaryMap());
		
		ProMTable previewTable = new ProMTable(tableModel);
		
		DefaultTableCellRenderer centerRenderer = new DefaultTableCellRenderer();
		centerRenderer.setHorizontalAlignment( JLabel.CENTER );
		previewTable.getTable().setDefaultRenderer(Object.class, centerRenderer);
		
		previewTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
		Enumeration<TableColumn> columns = previewTable.getColumnModel().getColumns();
		while (columns.hasMoreElements()) {
			final TableColumn column = columns.nextElement();
			column.setPreferredWidth(130);
			
			column.setCellEditor(new DefaultCellEditor(new JTextField()) {

				protected void fireEditingStopped() {
					this.cancelCellEditing();
					super.fireEditingStopped();
				}

				protected void fireEditingCanceled() {
					super.fireEditingCanceled();
				}

			});
		}
		previewTable.getColumnModel().setColumnSelectionAllowed(false);
		previewTable.getTableHeader().setReorderingAllowed(false);
		
		//
		ProMScrollPane mainScrollPane = new ProMScrollPane(previewTable);
		mainScrollPane.setHorizontalScrollBarPolicy( ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		statisticPanel.setPreferredSize(new Dimension(5 * splitPanel.getWidth() / 6, splitPanel.getHeight()));
		
		splitPanel.setLeftComponent(mainScrollPane);
		splitPanel.setRightComponent(statisticPanel);
		
		Dimension minimumSize = new Dimension(200, 200);
		mainScrollPane.setMinimumSize(minimumSize);
		statisticPanel.setMinimumSize(minimumSize);
		
		statisticPanel.setBackground(new Color(192,192,192));
		splitPanel.setResizeWeight(1.0);		
		return splitPanel;
	}	
	protected Component createLegendPanel() {
		JPanel legendPanel = new JPanel();
		legendPanel.setBorder(BorderFactory.createEmptyBorder());
		legendPanel.setBackground(new Color(192, 192, 192));
		TableLayout layout = new TableLayout(new double[][] { { 0.10, TableLayout.FILL }, {} });
		layout.insertRow(0, 0.2);
		legendPanel.setLayout(layout);
		
		List<String> dataObjects = new ArrayList<String>();
		dataObjects.add(AllDataObjects);
		dataObjects.addAll(data.getDataObjects());
		
		dataObjectComboBox = new ProMComboBox<String>(dataObjects);
		dataObjectComboBox.addActionListener(this);
		dataObjectComboBox.setPreferredSize(new Dimension(200, dataObjectComboBox.getPreferredSize().height));
//		legendPanel.add(dataObjectComboBox);
		
		alignmentMoveTypeComboBox = new ProMComboBox<AlignmentMoveType>(AlignmentMoveType.values());
		alignmentMoveTypeComboBox.addActionListener(this);
		alignmentMoveTypeComboBox.setPreferredSize(new Dimension(200, alignmentMoveTypeComboBox.getPreferredSize().height));
//		legendPanel.add(alignmentMoveTypeComboBox);
		//******************************
		data.updateSummary((String) dataObjectComboBox.getSelectedItem(), (AlignmentMoveType)alignmentMoveTypeComboBox.getSelectedItem());
		SlickerFactory factory = SlickerFactory.instance();

		int row = 1;

		layout.insertRow(row, TableLayout.PREFERRED);
		JLabel legend = SlickerFactory.instance().createLabel("Filter Options");
		legend.setPreferredSize(new Dimension(200, dataObjectComboBox.getPreferredSize().height));
		legendPanel.add(legend, "0,1,1,1,l, c");
		row++;

		layout.insertRow(row, 0.2);

		layout.insertRow(row, TableLayout.PREFERRED);
		legendPanel.add(dataObjectComboBox, "1," + row + ",l, c");
		JLabel syncLbl = factory.createLabel("Selected Data Object: ");
		legendPanel.add(syncLbl, "0," + row++ + ",r, c");

		layout.insertRow(row, TableLayout.PREFERRED);
		legendPanel.add(alignmentMoveTypeComboBox, "1," + row + ",l, c");		
		
		JLabel moveInviLbl = factory.createLabel("Alignment Move Type: ");
		legendPanel.add(moveInviLbl, "0," + row++ + ",r, c");

		return legendPanel;
	}
	public class ScrollBar extends JScrollPane {
		private static final long serialVersionUID = 1L;

		public ScrollBar(JComponent component){
			super(component);
			setOpaque(true);
			setBackground(WidgetColors.PROPERTIES_BACKGROUND);
			getViewport().setOpaque(true);
			getViewport().setBackground(WidgetColors.PROPERTIES_BACKGROUND);
			setBorder(BorderFactory.createEmptyBorder());
			setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
			setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
			JScrollBar hBar = getHorizontalScrollBar();
			hBar.setUI(new SlickerScrollBarUI(hBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
					WidgetColors.COLOR_NON_FOCUS, 4, 12));
			hBar.setOpaque(true);
			hBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);

			
			JScrollBar vBar = getVerticalScrollBar();
			vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
					WidgetColors.COLOR_NON_FOCUS, 4, 12));
			vBar.setOpaque(true);
			vBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		}
	}
	public class SummaryTableModel extends AbstractTableModel{ 
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		//Two arrays used for the table data
	    List<String> columnNames;
	    String[][] data;
	    
	    public SummaryTableModel(List<String> operations, HashMap<String, int[]> summaryMap){
	    	columnNames = new ArrayList<String>();
	    	columnNames.add(0, "");
	    	for (String operation : operations) {
	    		columnNames.add("LA:" + operation);
	    		columnNames.add("IA:" + operation);
	    		columnNames.add("MA:" + operation);
			}
	    	updateDataObjects(summaryMap);
	    }
	    public void updateDataObjects(HashMap<String, int[]> summaryMap){
	    	data = new String[summaryMap.keySet().size()][13];
	    	int row = 0;
	    	for (String activity : summaryMap.keySet()) {
	    		int statistics[] = summaryMap.get(activity);
	    		data[row][0] = activity;
	    		for(int i = 0; i < statistics.length; i++)
	    			data[row][i + 1] = String.valueOf(statistics[i]);
	    		row++;
			}
	    }
	    
		public int getColumnCount() {
			return columnNames.size();
		}

		public int getRowCount() {
			return data.length;
		}
		@Override
        public String getColumnName(int column) {
            return columnNames.get(column);
        }
		public Object getValueAt(int rowIndex, int columnIndex) {
			return data[rowIndex][columnIndex];
		}		
	}
	public void actionPerformed(ActionEvent event) {
		System.out.println("Log");
		data.updateSummary((String)dataObjectComboBox.getSelectedItem(), (AlignmentMoveType)alignmentMoveTypeComboBox.getSelectedItem());
		tableModel.updateDataObjects(data.getSummaryMap());
		tableModel.fireTableDataChanged();		
	}
}
